@@ -182,5 +182,148 @@ describe Rufus::Scheduler::EveryJob do
       end
     end
   end
+
+  describe '#next_times' do
+
+    it 'returns the next n times' do
+
+      job = @scheduler.schedule_every '5m' do; end
+
+      times =
+        2.times.inject([ job.next_time ]) { |a|
+          a << job.next_time_from(a.last)
+          a }
+
+      expect(job.next_times(3)).to eq(times)
+    end
+
+    it 'takes first_at/in into account' do
+
+      job = @scheduler.schedule_every '5m', first_in: 3 do; end
+
+      times =
+        2.times.inject([ job.next_time ]) { |a|
+          a << job.next_time_from(a.last)
+          a }
+
+      expect(job.next_times(3)).to eq(times)
+
+      expect(times.first).to be_between(Time.now + 2.5, Time.now + 3.5)
+    end
+  end
+
+  describe '#resume' do
+
+    context 'discard_past: true' do
+
+      it 'skips the skipped times' do
+
+        #@scheduler.discard_past = true
+          # this is the default
+
+        job = @scheduler.schedule_every('1s') do; end
+        job.pause
+
+        expect(job.count).to eq(0)
+
+        sleep 2
+
+        job.resume
+
+        wait_until { job.count > 0 }
+
+        expect(job.count).to eq(1)
+      end
+
+      it 'skips the skipped times' do
+
+        @scheduler.discard_past = false
+
+        job = @scheduler.schedule_every('1s', discard_past: true) do; end
+        job.pause
+
+        expect(job.count).to eq(0)
+
+        sleep 2
+
+        job.resume
+
+        wait_until { job.count > 0 }
+
+        expect(job.count).to eq(1)
+      end
+
+      it 'skips the skipped times' do
+
+        @scheduler.discard_past = false
+
+        job = @scheduler.schedule_every('1s', discard_past: false) do; end
+        job.pause
+
+        expect(job.count).to eq(0)
+
+        sleep 2
+
+        job.resume(discard_past: true)
+
+        wait_until { job.count > 0 }
+
+        expect(job.count).to eq(1)
+      end
+    end
+
+    context 'discard_past: false' do
+
+      it 'triggers the skipped times' do
+
+        @scheduler.discard_past = false
+
+        job = @scheduler.schedule_every('1s') do; end
+        job.pause
+
+        expect(job.count).to eq(0)
+
+        sleep 2
+
+        job.resume
+
+        sleep 1
+
+        expect(job.count).to be > 1
+      end
+
+      it 'triggers the skipped times' do
+
+        job = @scheduler.schedule_every('1s', discard_past: false) do; end
+        job.pause
+
+        expect(job.count).to eq(0)
+
+        sleep 2
+
+        job.resume
+
+        sleep 1
+
+        expect(job.count).to be > 1
+      end
+
+      it 'triggers the skipped times' do
+
+        job = @scheduler.schedule_every('1s') do; end
+        job.pause
+
+        expect(job.count).to eq(0)
+
+        sleep 2
+
+        job.resume(discard_past: false)
+
+        sleep 1
+
+        expect(job.count).to be > 1
+      end
+    end
+  end
 end
 
