@@ -45,9 +45,7 @@ describe Rufus::Scheduler::Job do
 
       job = @scheduler.schedule_in '0s' do; end
 
-      sleep 0.4
-
-      expect(job.last_time).not_to eq(nil)
+      wait_until { job.last_time }
     end
   end
 
@@ -76,12 +74,9 @@ describe Rufus::Scheduler::Job do
           sleep(1)
         end
 
-      sleep 0.4
+      wait_until { job.threads.size > 0 }
 
-      expect(job.threads.size).to eq(1)
-
-      t = job.threads.first
-      expect(t[:rufus_scheduler_job]).to eq(job)
+      expect(job.threads.first[:rufus_scheduler_job]).to eq(job)
     end
   end
 
@@ -143,9 +138,7 @@ describe Rufus::Scheduler::Job do
 
       job = @scheduler.in('0s', :job => true) { sleep(1) }
 
-      sleep 0.4
-
-      expect(job.running?).to eq(true)
+      wait_until { job.running? }
     end
   end
 
@@ -171,13 +164,14 @@ describe Rufus::Scheduler::Job do
 
       job = @scheduler.schedule_interval('0.4s') { sleep(10) }
 
-      sleep 1
+      wait_until { job.running? }
 
       expect(job.running?).to eq(true)
       expect(job.scheduled?).to eq(true)
     end
 
     it 'returns false if job is unscheduled' do
+
       job = @scheduler.schedule_interval('0.1s') { sleep 0.1 }
       job.unschedule
 
@@ -200,7 +194,7 @@ describe Rufus::Scheduler::Job do
         end
       job.call
 
-      sleep 0.8
+      wait_until { counter > 1 }
 
       expect(counter).to eq(2)
     end
@@ -264,9 +258,7 @@ describe Rufus::Scheduler::Job do
             job[:counter] += 1
           end
 
-        sleep 3
-
-        expect(job[:counter]).to be > 1
+        wait_until { job[:counter] && job[:counter] > 1 }
       end
     end
 
@@ -295,10 +287,23 @@ describe Rufus::Scheduler::Job do
         job = @scheduler.schedule_in '1s' do; end
         job[:x] = :y
 
+        expect(job.key?(:a)).to eq(false)
         expect(job.key?(:x)).to eq(true)
       end
     end
 
+    describe '#has_key?' do
+
+      it 'returns true if there is an entry with the given key' do
+
+        job = @scheduler.schedule_in '1s' do; end
+        job[:x] = :y
+
+        expect(job.has_key?(:a)).to eq(false)
+        expect(job.has_key?(:x)).to eq(true)
+      end
+    end
+
     describe '#keys' do
 
       it 'returns the array of keys of the job-local variables' do
@@ -311,6 +316,135 @@ describe Rufus::Scheduler::Job do
         expect(job.keys.sort_by { |k| k.to_s }).to eq([ 123, 'hello', :x ])
       end
     end
+
+    describe '#values' do
+
+      it 'returns the array of values of the job-local variables' do
+
+        job = @scheduler.schedule_in '1s' do; end
+        job[:x] = :y
+        job['hello'] = :z
+        job[123] = {}
+
+        expect(job.values).to eq([ :y, :z, {} ])
+      end
+    end
+
+    describe '#entries' do
+
+      it 'returns the array of entry pairs of the job-local variables' do
+
+        job = @scheduler.schedule_in '1s' do; end
+        job[:x] = :y
+        job['hello'] = :z
+        job[123] = {}
+
+        expect(job.entries).to eq([ [ :x, :y ], [ 'hello', :z ], [ 123, {} ] ])
+      end
+    end
+
+    it 'can be set at job scheduling time' do
+
+      j0 = @scheduler.schedule_in '1s', locals: { a: :alpha } do; end
+      j1 = @scheduler.schedule_in '1s', l: { a: :aleph } do; end
+
+      expect(j0[:a]).to eq(:alpha)
+      expect(j1[:a]).to eq(:aleph)
+    end
+
+    it 'is accessible to pre, post, and around hooks before first run' do
+
+      value = rand
+
+      job =
+        @scheduler.schedule_in('0.01s', l: { one: value }, times: 1) do
+          $out << "in the job #{value}"
+        end
+
+      $out = []
+
+      def @scheduler.on_pre_trigger(job)
+        $out << "pre #{job[:one]}"
+      end
+      def @scheduler.on_post_trigger(job)
+        $out << "post #{job[:one]}"
+      end
+      def @scheduler.around_trigger(job)
+        $out << "around-pre #{job[:one]}"
+        yield
+        $out << "around-post #{job[:one]}"
+      end
+
+      wait_until { $out.size > 4 }
+
+      expect($out).to eq([
+        "pre #{value}",
+        "around-pre #{value}",
+        "in the job #{value}",
+        "around-post #{value}",
+        "post #{value}"
+      ])
+    end
+
+    describe '#name' do
+
+      it 'returns the job name' do
+
+        j = @scheduler.schedule_in '10d', name: 'alice' do; end
+
+        expect(j.name).to eq('alice')
+      end
+    end
+
+    describe '#location' do
+
+      it 'returns the job location in code' do
+
+        j = @scheduler.schedule_in '10d', name: 'alice' do; end
+
+        l = j.location
+
+        expect(l[0]).to match(/\/rufus-scheduler\/spec\/job_spec\.rb$/)
+        expect(l[1]).to eq(__LINE__ - 5)
+      end
+
+      class InstanceHandler
+        def call(job, time); end
+      end
+
+      it 'returns the right location for a callable instance job' do
+
+        j = @scheduler.schedule_in '10d', InstanceHandler
+
+        l = j.source_location
+
+        expect(l[0]).to match(/\/rufus-scheduler\/spec\/job_spec\.rb$/)
+        expect(l[1]).to eq(__LINE__ - 10)
+      end
+
+      it 'returns the right location for a callable class job' do
+
+        j =
+          @scheduler.schedule_in('10h', Class.new do
+            def call; end
+          end)
+
+        l = j.source_location
+
+        expect(l[0]).to match(/\/rufus-scheduler\/spec\/job_spec\.rb$/)
+        expect(l[1]).to eq(__LINE__ - 6)
+      end
+    end
+
+    describe '#locals' do
+
+      it 'returns the locals hash, as is' do
+
+        j = @scheduler.schedule_in '1s', locals: { a: :aa, b: :bb } do; end
+
+        expect(j.locals).to eq(a: :aa, b: :bb)
+      end
+    end
   end
 
   context ':tag / :tags => [ t0, t1 ]' do
@@ -401,7 +535,7 @@ describe Rufus::Scheduler::Job do
             sleep(5)
           end
 
-        sleep 3
+        wait_until { job.threads.size > 0 }
 
         expect(job.threads.size).to eq(1)
       end
@@ -423,7 +557,7 @@ describe Rufus::Scheduler::Job do
             sleep(3)
           end
 
-        sleep 0.7
+        wait_until { j0.threads.size + j1.threads.size > 0 }
 
         if j0.threads.any?
           expect(j0.threads.size).to eq(1)
@@ -446,7 +580,7 @@ describe Rufus::Scheduler::Job do
         j0 = @scheduler.in('0s', :job => true, :mutex => m) { sleep(3) }
         j1 = @scheduler.in('0s', :job => true, :mutex => m) { sleep(3) }
 
-        sleep 0.7
+        wait_until { j0.threads.size + j1.threads.size > 0 }
 
         if j0.threads.any?
           expect(j0.threads.size).to eq(1)
@@ -473,7 +607,7 @@ describe Rufus::Scheduler::Job do
             sleep(3)
           end
 
-        sleep 0.7
+        wait_until { j0.threads.size + j1.threads.size > 0 }
 
         if j0.threads.any?
           expect(j0.threads.size).to eq(1)
@@ -565,9 +699,9 @@ describe Rufus::Scheduler::Job do
         sleep 0.9
       end
 
-      sleep 2
-
-      expect($stderr.string).to match(/Rufus::Scheduler::TimeoutError/)
+      #wait_until { $stderr.string.match?(/Rufus::Scheduler::TimeoutError/) }
+        # no worky on older Rubies... so
+      wait_until { $stderr.string.match(/Rufus::Scheduler::TimeoutError/) }
     end
 
     it 'does not prevent a repeat job from recurring' do
@@ -579,9 +713,24 @@ describe Rufus::Scheduler::Job do
         sleep 0.9
       end
 
-      sleep 3
+      wait_until { counter > 1 }
+    end
+  end
 
-      expect(counter).to be > 1
+  context 'discard_past: true/false' do
+
+    # specified in spec/job_repeat_spec.rb
+  end
+
+  context 'name: / n:' do
+
+    it 'sets the job name' do
+
+      j0 = @scheduler.schedule_in '10d', name: 'Alfred' do; end
+      j1 = @scheduler.schedule_in '11d', n: 'Alberich' do; end
+
+      expect(j0.name).to eq('Alfred')
+      expect(j1.name).to eq('Alberich')
     end
   end
 
@@ -599,7 +748,7 @@ describe Rufus::Scheduler::Job do
       it 'keeps track of how long the work was upon last trigger' do
 
         job =
-          @scheduler.schedule_in '0.5s'  do
+          @scheduler.schedule_in '0.5s' do
             sleep 0.7
           end
 
@@ -631,7 +780,7 @@ describe Rufus::Scheduler::Job do
         sleep 4.6
 
         expect(job.last_work_time).to be >= 0.08
-        expect(job.last_work_time).to be < 0.099
+        expect(job.last_work_time).to be < 0.0999
         expect(job.mean_work_time).to be > 0.05
         expect(job.mean_work_time).to be < 0.07
       end
