@@ -35,21 +35,21 @@ describe Rufus::Scheduler do
       )
     end
 
-    it 'accepts a :frequency => integer option' do
+    it 'accepts a frequency: 2 (int) option' do
 
       scheduler = Rufus::Scheduler.new(:frequency => 2)
 
       expect(scheduler.frequency).to eq(2)
     end
 
-    it 'accepts a :frequency => "2h1m" option' do
+    it 'accepts a frequency: "2h1m" option' do
 
       scheduler = Rufus::Scheduler.new(:frequency => '2h1m')
 
       expect(scheduler.frequency).to eq(3600 * 2 + 60)
     end
 
-    it 'accepts a :thread_name option' do
+    it 'accepts a thread_name: option' do
 
       scheduler = Rufus::Scheduler.new(:thread_name => 'oliphant')
 
@@ -63,19 +63,31 @@ describe Rufus::Scheduler do
     #  scheduler.min_work_threads.should == 9
     #end
 
-    it 'accepts a :max_work_threads option' do
+    it 'accepts a max_work_threads: option' do
 
       scheduler = Rufus::Scheduler.new(:max_work_threads => 9)
 
       expect(scheduler.max_work_threads).to eq(9)
     end
+
+    it 'accepts a discard_past: option' do
+
+      scheduler = Rufus::Scheduler.new
+
+      expect(scheduler.discard_past).to eq(true)
+
+      scheduler = Rufus::Scheduler.new(discard_past: false)
+
+      expect(scheduler.discard_past).to eq(false)
+    end
   end
 
   before :each do
     @scheduler = Rufus::Scheduler.new
   end
   after :each do
-    @scheduler.shutdown
+    @scheduler.shutdown if @scheduler.uptime
+      # don't shutdown an already shut down scheduler.
   end
 
   describe 'a schedule method' do
@@ -85,7 +97,7 @@ describe Rufus::Scheduler do
       j = nil
       job = @scheduler.schedule_in('0s') { |jj| j = jj }
 
-      sleep 0.4
+      wait_until { j }
 
       expect(j).to eq(job)
     end
@@ -95,7 +107,7 @@ describe Rufus::Scheduler do
       t = nil
       @scheduler.schedule_in('0s') { |jj, tt| t = tt }
 
-      sleep 0.4
+      wait_until { t }
 
       expect(t.class).to eq(EtOrbi::EoTime)
     end
@@ -116,7 +128,7 @@ describe Rufus::Scheduler do
 
       @scheduler.schedule_in('0s', mh)
 
-      sleep 0.4
+      wait_until { mh.counter > 0 }
 
       expect(mh.counter).to eq(1)
     end
@@ -135,7 +147,7 @@ describe Rufus::Scheduler do
 
       job = @scheduler.schedule_in('0s', MyOtherHandler.new)
 
-      sleep 0.4
+      wait_until { job.handler.counter > 0 }
 
       expect(job.handler.counter).to eq(1)
     end
@@ -150,7 +162,7 @@ describe Rufus::Scheduler do
           end
         end)
 
-      sleep 0.4
+      wait_until { job.handler.value }
 
       expect(job.handler.value).to eq(7)
     end
@@ -177,7 +189,7 @@ describe Rufus::Scheduler do
 
       @scheduler.in(t) { seen = true }
 
-      sleep 0.1 while seen != true
+      wait_until { seen }
     end
 
     it 'accepts point in time and duration indifferently (#at)' do
@@ -188,7 +200,7 @@ describe Rufus::Scheduler do
 
       @scheduler.at(t) { seen = true }
 
-      sleep 0.1 while seen != true
+      wait_until { seen }
     end
   end
 
@@ -290,9 +302,9 @@ describe Rufus::Scheduler do
 
     it 'returns the uptime as a human readable string' do
 
-      sleep 1
+      sleep(1)
 
-      expect(@scheduler.uptime_s).to match(/^[12]s\d+$/)
+      expect(@scheduler.uptime_s).to match(/\A[12]\.\d+s\z/)
     end
   end
 
@@ -312,6 +324,47 @@ describe Rufus::Scheduler do
     end
   end
 
+  describe '#join(limit)' do
+
+    it 'joins the scheduler thread' do
+
+      t =
+        Thread.new do
+          Thread.current['r'] = @scheduler.join(1)
+          Thread.current['a'] = 'over'
+        end
+
+      expect(t['r']).to eq(nil)
+      expect(t['a']).to eq(nil)
+
+      @scheduler.shutdown
+
+      sleep(1)
+
+      expect(t['r'].class).to eq(Thread)
+      expect(t['a']).to eq('over')
+    end
+
+    it 'times out' do
+
+      a = []
+
+      @scheduler.in(0.010) do
+        a << 0
+        sleep 3
+        a << 1
+      end
+
+      t0 = monow
+
+      r = @scheduler.join(1)
+
+      expect(monow - t0).to be < 1.5
+      expect(a).to eq([ 0 ])
+      expect(r).to eq(nil)
+    end
+  end
+
   describe '#job(job_id)' do
 
     it 'returns nil if there is no corresponding Job instance' do
@@ -364,7 +417,7 @@ describe Rufus::Scheduler do
         sleep(2)
       end
 
-      sleep 0.4
+      wait_until { @scheduler.threads.size == 2 }
 
       expect(@scheduler.threads.size).to eq(2)
     end
@@ -416,7 +469,7 @@ describe Rufus::Scheduler do
           sleep 1
         end
 
-      sleep 0.49
+      wait_until { @scheduler.work_threads(:active).size >= 1 }
 
       expect(@scheduler.work_threads(:active).size).to eq(1)
 
@@ -437,7 +490,7 @@ describe Rufus::Scheduler do
           sleep(1)
         end
 
-      sleep 0.4
+      wait_until { @scheduler.work_threads(:active).empty? }
 
       expect(scheduler.work_threads(:active)).to eq([])
 
@@ -509,7 +562,7 @@ describe Rufus::Scheduler do
           sleep(1)
         end
 
-      sleep 0.4
+      wait_until { @scheduler.running_jobs.size >= 1 }
 
       expect(job.running?).to eq(true)
       expect(@scheduler.running_jobs).to eq([ job ])
@@ -522,7 +575,7 @@ describe Rufus::Scheduler do
           sleep(5)
         end
 
-      sleep 1.5
+      wait_until { @scheduler.running_jobs.size >= 1 }
 
       expect(job.running?).to eq(true)
       expect(@scheduler.running_jobs).to eq([ job ])
@@ -540,17 +593,17 @@ describe Rufus::Scheduler do
         sleep 3
       end
 
-      sleep 1
+      wait_until { @scheduler.running_jobs.size > 1 }
 
-      expect(@scheduler.running_jobs(:tag => 't0').map(&:original)).to eq(
-        %w[ 0.1s ]
-      )
-      expect(@scheduler.running_jobs(:tag => 't1').map(&:original)).to eq(
-        %w[ 0.2s ]
-      )
-      expect(@scheduler.running_jobs(:tags => %w[ t0 t1 ]).map(&:original)).to eq(
-        []
-      )
+      expect(
+        @scheduler.running_jobs(:tag => 't0').map(&:original)
+      ).to eq(%w[ 0.1s ])
+      expect(
+        @scheduler.running_jobs(:tag => 't1').map(&:original)
+      ).to eq(%w[ 0.2s ])
+      expect(
+        @scheduler.running_jobs(:tags => %w[ t0 t1 ]).map(&:original)
+      ).to eq([])
     end
   end
 
@@ -683,22 +736,129 @@ describe Rufus::Scheduler do
 
     it 'shuts down and blocks until all the jobs ended their current runs' do
 
-      counter = 0
+      seen = false
 
-      @scheduler.in '0s' do
-        sleep 1
-        counter = counter + 1
-      end
+      job =
+        @scheduler.schedule_in '0s' do
+          sleep 1
+          seen = true
+        end
 
-      sleep 0.4
+      wait_until { job.threads.first }
+
+      expect(seen).to eq(false)
 
       @scheduler.shutdown(:wait)
 
-      expect(counter).to eq(1)
+      expect(seen).to eq(true)
       expect(@scheduler.uptime).to eq(nil)
       expect(@scheduler.running_jobs).to eq([])
       expect(@scheduler.threads).to eq([])
     end
+
+    it 'does not mind being called from a scheduler job (gh-304)' do
+
+      seen = []
+
+      job0 =
+        @scheduler.schedule_in '0s' do
+          seen << :job0a
+          sleep 3
+          seen << :job0b
+        end
+
+      sleep 0.300
+
+      job1 =
+        @scheduler.schedule_in '0s' do
+          seen << :job1a
+          @scheduler.shutdown(:wait)
+          seen << :job1b
+        end
+
+      t0 = monow
+
+      wait_until { @scheduler.uptime == nil }
+
+      expect(monow - t0).to be_between(2.5, 4.0)
+
+      expect(seen.take(3)).to eq([ :job0a, :job1a, :job0b ])
+
+      wait_until { @scheduler.threads == [] }
+    end
+  end
+
+  describe '#shutdown(wait: n)' do
+
+    it 'waits no more than n seconds' do
+
+      seen = []
+
+      job0 =
+        @scheduler.schedule_in '0s' do
+          seen << :job0a
+          sleep 100
+          seen << :job0b
+        end
+
+      sleep 0.300
+
+      job1 =
+        @scheduler.schedule_in '0s' do
+          seen << :job1a
+          sleep 4
+          seen << :job1b
+        end
+
+      wait_until { seen.include?(:job1a) }
+
+      expect(seen).to eq([ :job0a, :job1a ])
+
+      t0 = monow
+
+      @scheduler.shutdown(wait: 2)
+
+      expect(monow - t0).to be_between(2.0, 3.0)
+
+      expect(job0.threads.size).to eq(0)
+      expect(job1.threads.size).to eq(0)
+
+      expect(@scheduler.uptime).to eq(nil)
+      expect(@scheduler.running_jobs).to eq([])
+      expect(@scheduler.threads).to eq([])
+    end
+
+    it 'does not mind being called from a scheduler job (gh-304)' do
+
+      seen = []
+
+      job0 =
+        @scheduler.schedule_in '0s' do
+          seen << :job0a
+          sleep 5
+          seen << :job0b
+        end
+
+      sleep 0.300
+
+      job1 =
+        @scheduler.schedule_in '0s' do
+          seen << :job1a
+          t0 = monow
+          @scheduler.shutdown(wait: 2)
+          seen << (monow - t0)
+        end
+
+      wait_until { seen.size > 2 }
+
+      expect(seen[0, 2]).to eq([ :job0a, :job1a ])
+      expect(seen[2]).to be_between(2.0, 3.0)
+
+      expect(@scheduler.uptime).to eq(nil)
+      expect(@scheduler.running_jobs).to eq([])
+
+      wait_until { @scheduler.threads == [] }
+    end
   end
 
   describe '#shutdown(:kill)' do
@@ -720,6 +880,13 @@ describe Rufus::Scheduler do
 
       @scheduler.shutdown(:kill)
 
+      wait_until { @scheduler.threads.empty? }
+
+      expect(counter).to eq(0)
+      expect(@scheduler.uptime).to eq(nil)
+      expect(@scheduler.running_jobs).to eq([])
+      expect(@scheduler.threads).to eq([])
+
       sleep 1.4
 
       expect(counter).to eq(0)
@@ -727,6 +894,32 @@ describe Rufus::Scheduler do
       expect(@scheduler.running_jobs).to eq([])
       expect(@scheduler.threads).to eq([])
     end
+
+    it 'does not mind being called from a scheduler job (gh-304)' do
+
+      seen = []
+
+      job0 =
+        @scheduler.schedule_in '0s' do
+          seen << :job0a
+          sleep 5
+          seen << :job0b
+        end
+
+      sleep 0.300
+
+      job1 =
+        @scheduler.schedule_in '0s' do
+          seen << :job1a
+          @scheduler.shutdown(:kill)
+          seen << :job1b
+        end
+
+      wait_until { seen.include?(:job1a) }
+      expect(@scheduler.uptime).to eq(nil)
+      wait_until { @scheduler.running_jobs.empty? }
+      wait_until { @scheduler.threads == [] }
+    end
   end
 
   describe '#pause' do
@@ -756,6 +949,51 @@ describe Rufus::Scheduler do
 
       expect(job.last_time).not_to eq(nil)
     end
+
+    context 'discard_past:' do
+
+      context 'by default' do
+
+        it 'discards the past' do
+
+          #@scheduler.discard_past = true
+            # this is the default
+
+          job = @scheduler.schedule_every('1s') do; end
+          @scheduler.pause
+
+          expect(job.count).to eq(0)
+
+          sleep 2
+
+          @scheduler.resume
+
+          wait_until { job.count > 0 }
+
+          expect(job.count).to eq(1)
+        end
+      end
+
+      context 'false' do
+
+        it 'prevents discarding the past' do
+
+          #@scheduler.discard_past = true
+            # this is the default
+
+          job = @scheduler.schedule_every('1s') do; end
+          @scheduler.pause
+
+          expect(job.count).to eq(0)
+
+          sleep 2
+
+          @scheduler.resume(discard_past: false)
+
+          wait_until { job.count > 1 }
+        end
+      end
+    end
   end
 
   describe '#paused?' do
@@ -829,9 +1067,7 @@ describe Rufus::Scheduler do
         sleep 2
       end
 
-      sleep 0.4
-
-      expect(@scheduler.jobs.size).to eq(1)
+      wait_until { @scheduler.jobs.size > 0 }
     end
 
     it 'does not return unscheduled jobs' do
@@ -967,7 +1203,7 @@ describe Rufus::Scheduler do
           $out << job.id
         end
 
-      sleep 0.7
+      wait_until { $out.size > 1 }
 
       expect($out).to eq([ "pre #{job_id}", job_id ])
     end
@@ -984,7 +1220,7 @@ describe Rufus::Scheduler do
 
       @scheduler.in '0.5s' do; end
 
-      sleep 0.7
+      wait_until { $tt }
 
       expect($tt.class).to eq(EtOrbi::EoTime)
       expect($tt).to be > start
@@ -1007,7 +1243,7 @@ describe Rufus::Scheduler do
             $out << job.id
           end
 
-        sleep 0.7
+        wait_until { $out.size > 0 }
 
         expect($out).to eq([ "pre #{job_id}" ])
       end
@@ -1029,12 +1265,85 @@ describe Rufus::Scheduler do
           $out << job.id
         end
 
-      sleep 0.7
+      wait_until { $out.size > 1 }
 
       expect($out).to eq([ job_id, "post #{job_id}" ])
     end
   end
 
+  describe '#around_trigger' do
+
+    it 'functions normally if no around_trigger is defined' do
+
+      $out = []
+
+      job_id =
+        @scheduler.in '0.5s' do |job|
+          $out << job.id
+        end
+
+      wait_until { $out.size > 0 }
+
+      expect($out).to eq([ job_id ])
+    end
+
+    it 'wraps the job' do
+
+      $out = []
+
+      def @scheduler.around_trigger(job)
+        $out << "pre #{job.id}"
+        yield
+        $out << "post #{job.id}"
+      end
+
+      job_id =
+        @scheduler.in '0.5s' do |job|
+          $out << job.id
+        end
+
+      wait_until { $out.size > 2 }
+
+      expect($out).to eq([ "pre #{job_id}", job_id, "post #{job_id}" ])
+    end
+
+    it 'does not block (runs in job thread)' do
+
+      $out = []
+
+      def @scheduler.around_trigger(job)
+        $out << "pre #{job.id}"
+        yield
+        $out << "post #{job.id}"
+      end
+
+      # using Thread.pass to space out job scheduling/triggering
+      # so that $out.take(3) becomes predictable
+
+      job_id1 =
+        @scheduler.in '0.1s' do |job|
+          sleep 0.5
+          $out << job.id
+        end
+
+      Thread.pass
+
+      job_id2 =
+        @scheduler.in '0.2s' do |job|
+          Thread.pass
+          $out << job.id
+        end
+
+      wait_until { $out.size > 2 }
+
+      expect($out.take(3)).to eq([
+        "pre #{job_id1}",
+        "pre #{job_id2}",
+        job_id2
+      ])
+    end
+  end
+
   #--
   # misc
   #++
